Package: MARSS
Type: Package
Title: Multivariate Autoregressive State-Space Modeling
Version: 3.11.10
Date: 2025-09-10
Depends: R (>= 3.5.0)
Imports: generics (>= 0.1.2), graphics, grDevices, KFAS (>= 1.0.1),
        mvtnorm, nlme, stats, utils
Suggests: forecast, ggplot2, Hmisc, knitr, lme4, maps, marssTMB, tidyr
Authors@R: 
    c(person("Elizabeth Eli", "Holmes", , "eli.holmes@noaa.gov", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-9128-8393")),
      person("Eric J.", "Ward", , "eric.ward@noaa.gov", role = c("aut"),
           comment = c(ORCID = "0000-0002-4359-0296")),
      person("Mark D.", "Scheuerell", , "scheuerl@uw.edu", role = c("aut"),
           comment = c(ORCID = "0000-0002-8284-1254")),
      person("Kellie", "Wills", , "wills.kellie@gmail.com", role = c("aut"))
      )
Description: The MARSS package provides maximum-likelihood parameter
    estimation for constrained and unconstrained linear multivariate autoregressive
    state-space (MARSS) models, including partially deterministic models. MARSS models are a class 
    of dynamic linear model (DLM) and vector autoregressive model (VAR)
    model. Fitting available via Expectation-Maximization (EM), BFGS (using optim), and 'TMB'
    (using the 'marssTMB' companion package). Functions are provided for parametric and 
    innovations bootstrapping, Kalman filtering and smoothing, model selection criteria including
    bootstrap AICb, confidences intervals via the Hessian approximation or bootstrapping, and all
    conditional residual types. See the user guide for examples of dynamic factor analysis, 
    dynamic linear models, outlier and shock detection, and multivariate AR-p models.  
    Online workshops (lectures, eBook, and computer labs) 
    at <https://atsa-es.github.io/>.
License: GPL-2
LazyData: yes
BuildVignettes: yes
ByteCompile: TRUE
URL: https://atsa-es.github.io/MARSS/
BugReports: https://github.com/atsa-es/MARSS/issues
RoxygenNote: 7.3.3
Encoding: UTF-8
VignetteBuilder: knitr, utils
Additional_repositories: https://atsa-es.r-universe.dev
NeedsCompilation: no
Packaged: 2025-09-09 22:16:55 UTC; eli.holmes
Author: Elizabeth Eli Holmes [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9128-8393>),
  Eric J. Ward [aut] (ORCID: <https://orcid.org/0000-0002-4359-0296>),
  Mark D. Scheuerell [aut] (ORCID:
    <https://orcid.org/0000-0002-8284-1254>),
  Kellie Wills [aut]
Maintainer: Elizabeth Eli Holmes <eli.holmes@noaa.gov>
Repository: CRAN
Date/Publication: 2025-09-10 07:00:09 UTC
Built: R 4.6.0; ; 2026-04-26 01:10:23 UTC; unix
